@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  * panel-action-button.c: panel "Action Button" module
  *
  * Copyright (C) 2002 Sun Microsystems, Inc.
@@ -36,7 +37,6 @@
 #include "panel-config-global.h"
 #include "panel-gconf.h"
 #include "panel-profile.h"
-#include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
 #include "panel-force-quit.h"
 #include "panel-util.h"
@@ -91,13 +91,22 @@ screensaver_properties_enabled (void)
 	    panel_lockdown_get_disable_lock_screen ())
 		return FALSE;
 
-	return TRUE;
+	return panel_lock_screen_action_available ("prefs");
 }
 
 static gboolean
 screensaver_enabled (void)
 {
-  return !panel_lockdown_get_disable_lock_screen ();
+	if (panel_lockdown_get_disable_lock_screen ())
+		return FALSE;
+
+	return panel_lock_screen_action_available ("lock");
+}
+
+static gboolean
+panel_action_lock_is_disabled (void)
+{
+	return !screensaver_enabled ();
 }
 
 static void
@@ -129,7 +138,7 @@ panel_action_lock_setup_menu (PanelActionButton *button)
 
 	panel_applet_add_callback (button->priv->info,
 				   "prefs",
-				   NULL,
+				   GTK_STOCK_PROPERTIES,
 				   _("_Properties"),
 				   screensaver_properties_enabled);
 }
@@ -138,24 +147,11 @@ static void
 panel_action_lock_invoke_menu (PanelActionButton *button,
 			       const char *callback_name)
 {
-	char *command = NULL;
+	g_return_if_fail (PANEL_IS_ACTION_BUTTON (button));
+	g_return_if_fail (callback_name != NULL);
 
-	if (!strcmp (callback_name, "prefs"))
-		command = g_strdup ("xscreensaver-demo");
-
-	else if (!strcmp (callback_name, "activate") ||
-		 !strcmp (callback_name, "lock") ||
-		 !strcmp (callback_name, "exit") ||
-		 !strcmp (callback_name, "restart"))
-		command = g_strdup_printf ("xscreensaver-command -%s", callback_name);
-
-	if (command)
-		gdk_spawn_command_line_on_screen (
-			gtk_widget_get_screen (GTK_WIDGET (button)),
-			command,
-			NULL);
-
-	g_free (command);
+	panel_lock_screen_action (gtk_widget_get_screen (GTK_WIDGET (button)),
+				  callback_name);
 }
 
 /* Log Out
@@ -288,7 +284,6 @@ panel_action_connect_server (GtkWidget *widget)
 
 typedef struct {
 	PanelActionButtonType   type;
-	char                   *stock_icon;
 	char                   *icon_name;
 	char                   *text;
 	char                   *tooltip;
@@ -306,12 +301,11 @@ typedef struct {
 static PanelAction actions [] = {
 	{
 		PANEL_ACTION_NONE,
-		NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL
 	},
 	{
 		PANEL_ACTION_LOCK,
-		NULL,
 		"gnome-lockscreen",
 		N_("Lock Screen"),
 		N_("Protect your computer from unauthorized use"),
@@ -320,11 +314,10 @@ static PanelAction actions [] = {
 		panel_action_lock_screen,
 		panel_action_lock_setup_menu,
 		panel_action_lock_invoke_menu,
-		panel_lockdown_get_disable_lock_screen
+		panel_action_lock_is_disabled
 	},
 	{
 		PANEL_ACTION_LOGOUT,
-		NULL,
 		"gnome-logout",
 		N_("Log Out"),
 		N_("Log out of this session to log in as a different user or to shut down the computer"),
@@ -335,8 +328,7 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_RUN,
-		PANEL_STOCK_RUN,
-		NULL,
+		PANEL_RUN_ICON,
 		N_("Run Application..."),
 		N_("Run an Application by entering a command"),
 		"gospanel-555",
@@ -346,7 +338,6 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_SEARCH,
-		NULL,
 		"gnome-searchtool",
 		N_("Search for Files..."),
 		N_("Find files, folders, and documents on your computer"),
@@ -356,7 +347,6 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_SCREENSHOT,
-		NULL,
 		"applets-screenshooter",
 		N_("Take Screenshot..."),
 		N_("Take a screenshot of your desktop"),
@@ -366,8 +356,7 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_FORCE_QUIT,
-		PANEL_STOCK_FORCE_QUIT,
-		NULL,
+		PANEL_FORCE_QUIT_ICON,
 		N_("Force Quit"),
 		N_("Force a misbehaving application to quit"),
 		"gospanel-563",
@@ -377,7 +366,6 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_CONNECT_SERVER,
-		NULL,
 		"gnome-globe", //FIXME icon
 		N_("Connect to Server..."),
 		N_("Connect to a remote server"), //FIXME
@@ -408,14 +396,6 @@ panel_action_get_invoke (PanelActionButtonType type)
 	return G_CALLBACK (actions[type].invoke);
 }
 
-G_CONST_RETURN char*
-panel_action_get_stock_icon (PanelActionButtonType type)
-{
-	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, NULL);
-
-	return actions[type].stock_icon;
-}
-
 G_CONST_RETURN char*
 panel_action_get_icon_name (PanelActionButtonType type)
 {
@@ -667,8 +647,6 @@ panel_action_button_set_type (PanelActionButton     *button,
 
 	if (actions [type].icon_name != NULL)
 		button_widget_set_icon_name (BUTTON_WIDGET (button), actions [type].icon_name);
-	else
-		button_widget_set_stock_id (BUTTON_WIDGET (button), actions [type].stock_icon);
 
 	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
 			      _(actions [type].tooltip), NULL);
@@ -721,8 +699,6 @@ panel_action_button_style_set (PanelActionButton *button)
 {
 	if (actions [button->priv->type].icon_name != NULL)
 		button_widget_set_icon_name (BUTTON_WIDGET (button), actions [button->priv->type].icon_name);
-	else
-		button_widget_set_stock_id (BUTTON_WIDGET (button), actions [button->priv->type].stock_icon);
 }
 
 static void
@@ -938,11 +914,8 @@ panel_action_button_set_dnd_enabled (PanelActionButton *button,
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-		if (actions [button->priv->type].stock_icon != NULL)
-			gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
-							actions [button->priv->type].stock_icon);
 		/* FIXME: waiting for bug #116577
-		else
+		if (actions [button->priv->type].icon_name != NULL)
 		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
 						actions [button->priv->type].icon_name);
 						*/
