@@ -41,6 +41,7 @@
 #include "panel-profile.h"
 #include "panel-lockdown.h"
 #include "panel-stock-icons.h"
+#include "panel-typebuiltins.h"
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
@@ -52,70 +53,16 @@ struct _PanelMenuBarPrivate {
 	GtkWidget   *applications_item;
 	GtkWidget   *places_item;
 	GtkWidget   *desktop_item;
+
+	PanelOrientation orientation;
+};
+
+enum {
+	PROP_0,
+	PROP_ORIENTATION,
 };
 
 static GObjectClass *parent_class;
-static GtkWidget *panel_menu_bar_create_applications_menu (PanelMenuBar *menubar);
-
-static void
-panel_menu_bar_recreate_menus (PanelMenuBar *menubar)
-{
-	if (menubar->priv->applications_menu) {
-		gtk_widget_destroy (menubar->priv->applications_menu);
-		menubar->priv->applications_menu = panel_menu_bar_create_applications_menu (menubar);
-		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
-					       "menu_panel",
-					       menubar->priv->panel);
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->applications_item),
-					   menubar->priv->applications_menu);
-	}
-}
-
-static void
-panel_menu_bar_append_applications_menu (GtkWidget    *menu,
-					 PanelMenuBar *menubar)
-{
-	GtkWidget *item;
-
-	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-appending"))
-		return;
-
-	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-appending", NULL);
-
-	item = panel_menu_items_create_action_item (PANEL_ACTION_RUN);
-
-	if (item != NULL) {
-		gboolean  add_separator;
-		GList    *children;
-
-		children = gtk_container_get_children (GTK_CONTAINER (menu));
-
-		if (children != NULL) {
-			while (children->next != NULL)
-				children = children->next;
-			add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
-		}
-
-		if (add_separator)
-			add_menu_separator (menu);
-
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-}
-
-static GtkWidget *
-panel_menu_bar_create_applications_menu (PanelMenuBar *menubar)
-{
-	GtkWidget *applications_menu;
-
-	applications_menu = create_applications_menu ("applications.menu", NULL);
-
-	g_signal_connect (applications_menu, "show",
-			  G_CALLBACK (panel_menu_bar_append_applications_menu),
-			  menubar);
-
-	return applications_menu;
-}
 
 static void
 panel_menu_bar_instance_init (PanelMenuBar      *menubar,
@@ -127,7 +74,7 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 
 	menubar->priv->info = NULL;
 
-	menubar->priv->applications_menu = panel_menu_bar_create_applications_menu (menubar);
+	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
 
 	menubar->priv->applications_item = gtk_image_menu_item_new_with_label (_("Applications"));
 	image = gtk_image_new_from_icon_name (PANEL_GNOME_LOGO_ICON,
@@ -147,9 +94,50 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 	menubar->priv->desktop_item = panel_desktop_menu_item_new (FALSE, TRUE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->desktop_item);
+}
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_menus),
-				   menubar);
+static void
+panel_menu_bar_get_property (GObject	*object,
+			     guint	 prop_id,
+			     GValue	*value,
+			     GParamSpec *pspec)
+{
+	PanelMenuBar *menubar;
+
+	g_return_if_fail (PANEL_IS_MENU_BAR (object));
+
+	menubar = PANEL_MENU_BAR (object);
+
+	switch (prop_id) {
+	case PROP_ORIENTATION:
+		g_value_set_enum (value, menubar->priv->orientation);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+panel_menu_bar_set_property (GObject	  *object,
+			     guint	   prop_id,
+			     const GValue *value,
+			     GParamSpec	  *pspec)
+{
+	PanelMenuBar *menubar;
+
+	g_return_if_fail (PANEL_IS_MENU_BAR (object));
+
+	menubar = PANEL_MENU_BAR (object);
+
+	switch (prop_id) {
+	case PROP_ORIENTATION:
+		panel_menu_bar_set_orientation (menubar, g_value_get_enum (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
@@ -196,17 +184,6 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 	panel_menu_bar_change_background (PANEL_MENU_BAR (widget));
 }
 
-static void
-panel_menu_bar_finalize (GObject *object)
-{
-	PanelMenuBar *menubar = (PanelMenuBar *) object;
-
-	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_bar_recreate_menus),
-				      menubar);
-
-	parent_class->finalize (object);
-}
-
 static void
 panel_menu_bar_class_init (PanelMenuBarClass *klass)
 {
@@ -215,12 +192,24 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	gobject_class->finalize  = panel_menu_bar_finalize;
+	gobject_class->get_property = panel_menu_bar_get_property;
+        gobject_class->set_property = panel_menu_bar_set_property;
+
 	widget_class->parent_set = panel_menu_bar_parent_set;
 	widget_class->size_allocate = panel_menu_bar_size_allocate;
 
 	g_type_class_add_private (klass, sizeof (PanelMenuBarPrivate));
 
+	g_object_class_install_property (
+		gobject_class,
+		PROP_ORIENTATION,
+		g_param_spec_enum ("orientation",
+				   "Orientation",
+				   "The PanelMenuBar orientation",
+				   PANEL_TYPE_ORIENTATION,
+				   PANEL_ORIENTATION_TOP,
+				   G_PARAM_READWRITE));
+
 	gtk_rc_parse_string (
 		"style \"panel-menubar-style\"\n"
 		"{\n"
@@ -301,14 +290,24 @@ panel_menu_bar_load (PanelWidget *panel,
 				   GTK_STOCK_HELP,
 				   _("_Help"),
 				   NULL);
-	g_signal_connect_after (G_OBJECT (menubar), "focus-in-event",
+
+	if (panel_is_program_in_path ("gmenu-simple-editor")) {
+		panel_applet_add_callback (menubar->priv->info,
+					   "edit",
+					   NULL,
+					   _("_Edit Menus"),
+					   NULL);
+	}
+
+	g_signal_connect_after (menubar, "focus-in-event",
 				G_CALLBACK (gtk_widget_queue_draw), menubar);
-	g_signal_connect_after (G_OBJECT (menubar), "focus-out-event",
+	g_signal_connect_after (menubar, "focus-out-event",
 				G_CALLBACK (gtk_widget_queue_draw), menubar);
-	g_signal_connect_after (G_OBJECT (menubar), "expose-event",
+	g_signal_connect_after (menubar, "expose-event",
 				G_CALLBACK (panel_menu_bar_on_expose), menubar);
-	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
 	GTK_WIDGET_SET_FLAGS (menubar, GTK_CAN_FOCUS);
+
+	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
 }
 
 void
@@ -341,12 +340,28 @@ panel_menu_bar_invoke_menu (PanelMenuBar *menubar,
 	g_return_if_fail (PANEL_IS_MENU_BAR (menubar));
 	g_return_if_fail (callback_name != NULL);
 
-	if (strcmp (callback_name, "help"))
-		return;
-
 	screen = gtk_widget_get_screen (GTK_WIDGET (menubar));
 
-	panel_show_help (screen, "user-guide.xml", "gospanel-37");
+	if (!strcmp (callback_name, "help")) {
+		panel_show_help (screen, "user-guide.xml", "gospanel-37");
+
+	} else if (!strcmp (callback_name, "edit")) {
+		GError *error = NULL;
+
+		panel_launch_desktop_file ("gmenu-simple-editor.desktop",
+					   "gmenu-simple-editor",
+					   screen,
+					   &error);
+		if (error) {
+			panel_error_dialog (screen,
+					    "cannot_exec_gmenu-simple-editor", TRUE,
+					    _("Cannot execute '%s'"),
+					    "%s",
+					    "gmenu-simple-editor",
+					    error->message);
+			g_error_free (error);
+		}
+	}
 }
 
 void
@@ -377,67 +392,183 @@ panel_menu_bar_popup_menu (PanelMenuBar *menubar,
 				    gtk_menu_get_attach_widget (menu));
 }
 
-void panel_menu_bar_change_background (PanelMenuBar *menubar)
+static void
+panel_menu_bar_set_no_background (PanelMenuBar *menubar)
 {
-	GtkRcStyle       *rc_style;
-	GtkStyle         *style;
-	const PanelColor *color;
-	GdkGC            *gc;
-	GdkPixmap        *pixmap;
-	const GdkPixmap  *bg_pixmap;
+	GtkRcStyle *rc_style;
 
-	/* reset style */
 	gtk_widget_set_style (GTK_WIDGET (menubar), NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (GTK_WIDGET (menubar), rc_style);
 	gtk_rc_style_unref (rc_style);
 
-	switch (panel_background_get_type (&menubar->priv->panel->background)) {
+}
+
+static void
+panel_menu_bar_set_image_background (PanelMenuBar *menubar)
+{
+	const GdkPixmap  *bg_pixmap;
+	GdkPixmap        *pixmap;
+	GtkStyle         *style;
+	GdkGC            *gc;
+
+	bg_pixmap = panel_background_get_pixmap (&menubar->priv->panel->background);
+	if (!bg_pixmap)
+		return;
+
+	gc = gdk_gc_new (GTK_WIDGET (menubar)->window);
+	g_return_if_fail (GDK_IS_GC (gc));
+
+	pixmap = gdk_pixmap_new (GTK_WIDGET (menubar)->window,
+				 GTK_WIDGET (menubar)->allocation.width,
+				 GTK_WIDGET (menubar)->allocation.height,
+				 -1);
+
+	gdk_draw_drawable (GDK_DRAWABLE (pixmap),
+			   gc, 
+			   GDK_DRAWABLE (bg_pixmap),
+			   GTK_WIDGET (menubar)->allocation.x,
+			   GTK_WIDGET (menubar)->allocation.y,
+			   0, 0,
+			   GTK_WIDGET (menubar)->allocation.width,
+			   GTK_WIDGET (menubar)->allocation.height);
+
+	g_object_unref (gc);
+
+	style = gtk_style_copy (GTK_WIDGET (menubar)->style);
+	if (style->bg_pixmap[GTK_STATE_NORMAL])
+		g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+	style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+	gtk_widget_set_style (GTK_WIDGET (menubar), style);
+	g_object_unref (style);
+
+	g_object_unref (pixmap);
+}
+
+static void
+panel_menu_bar_set_color_background (PanelMenuBar *menubar)
+{
+	const PanelColor *color;
+
+	color = panel_background_get_color (&menubar->priv->panel->background);
+	if (color->alpha != 0xffff) {
+		panel_menu_bar_set_image_background (menubar);
+		return;
+	}
+
+	gtk_widget_modify_bg (GTK_WIDGET (menubar), GTK_STATE_NORMAL, &color->gdk);
+}
+
+void
+panel_menu_bar_change_background (PanelMenuBar *menubar)
+{
+	PanelBackgroundType type;
+
+	panel_menu_bar_set_no_background (menubar);
+
+	type = panel_background_get_type (&menubar->priv->panel->background);
+
+	switch (type) {
 	case PANEL_BACK_NONE:
 		break;
 	case PANEL_BACK_COLOR:
-		color = panel_background_get_color (&menubar->priv->panel->background);
-		if (color->alpha == 0xffff) {
-			gtk_widget_modify_bg (GTK_WIDGET (menubar),
-					      GTK_STATE_NORMAL, &(color->gdk));
-			break;
-		}
-		/* else, we have an image, so don't break */
+		panel_menu_bar_set_color_background (menubar);
+		break;
 	case PANEL_BACK_IMAGE:
-		bg_pixmap = panel_background_get_pixmap (&menubar->priv->panel->background);
-		if (!bg_pixmap)
-			return;
-
-		gc = gdk_gc_new (GTK_WIDGET (menubar)->window);
-		g_return_if_fail (GDK_IS_GC (gc));
-
-		pixmap = gdk_pixmap_new (GTK_WIDGET (menubar)->window,
-					 GTK_WIDGET (menubar)->allocation.width,
-					 GTK_WIDGET (menubar)->allocation.height,
-					 -1);
-
-		gdk_draw_drawable (GDK_DRAWABLE (pixmap),
-				   gc, 
-				   GDK_DRAWABLE (bg_pixmap),
-				   GTK_WIDGET (menubar)->allocation.x,
-				   GTK_WIDGET (menubar)->allocation.y,
-				   0, 0,
-				   GTK_WIDGET (menubar)->allocation.width,
-				   GTK_WIDGET (menubar)->allocation.height);
-
-		g_object_unref (gc);
-
-		style = gtk_style_copy (GTK_WIDGET (menubar)->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (GTK_WIDGET (menubar), style);
-		g_object_unref (style);
-
-		g_object_unref (pixmap);
+		panel_menu_bar_set_image_background (menubar);
 		break;
 	default:
 		g_assert_not_reached ();
 		break;
 	}
 }
+
+static void
+set_item_text_angle_and_alignment (GtkWidget *item,
+				   double     text_angle,
+				   float      xalign,
+				   float      yalign)
+{
+	GtkWidget *label;
+
+	label = GTK_BIN (item)->child;
+
+	gtk_label_set_angle (GTK_LABEL (label), text_angle);
+
+	gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
+}
+
+static void
+panel_menu_bar_update_orientation (PanelMenuBar *menubar)
+{
+	GtkPackDirection pack_direction;
+	double           text_angle;
+	float            text_xalign;
+	float            text_yalign;
+
+	pack_direction = GTK_PACK_DIRECTION_LTR;
+	text_angle = 0.0;
+	text_xalign = 0.0;
+	text_yalign = 0.5;
+
+	switch (menubar->priv->orientation) {
+	case PANEL_ORIENTATION_TOP:
+	case PANEL_ORIENTATION_BOTTOM:
+		break;
+	case PANEL_ORIENTATION_LEFT:
+		pack_direction = GTK_PACK_DIRECTION_BTT;
+		text_angle = 90.0;
+		text_xalign = 0.5;
+		text_yalign = 0.0;
+		break;
+	case PANEL_ORIENTATION_RIGHT:
+		pack_direction = GTK_PACK_DIRECTION_TTB;
+		text_angle = 270.0;
+		text_xalign = 0.5;
+		text_yalign = 0.0;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	gtk_menu_bar_set_pack_direction (GTK_MENU_BAR (menubar), pack_direction);
+	gtk_menu_bar_set_child_pack_direction (GTK_MENU_BAR (menubar), pack_direction);
+
+	set_item_text_angle_and_alignment (menubar->priv->applications_item,
+					   text_angle,
+					   text_xalign,
+					   text_yalign);
+	set_item_text_angle_and_alignment (menubar->priv->places_item,
+					   text_angle,
+					   text_xalign,
+					   text_yalign);
+	set_item_text_angle_and_alignment (menubar->priv->desktop_item,
+					   text_angle,
+					   text_xalign,
+					   text_yalign);
+}
+
+void
+panel_menu_bar_set_orientation (PanelMenuBar     *menubar,
+				PanelOrientation  orientation)
+{
+        g_return_if_fail (PANEL_IS_MENU_BAR (menubar));
+
+        if (menubar->priv->orientation == orientation)
+                return;
+
+        menubar->priv->orientation = orientation;
+
+        panel_menu_bar_update_orientation (menubar);
+
+        g_object_notify (G_OBJECT (menubar), "orientation");
+}
+
+PanelOrientation
+panel_menu_bar_get_orientation (PanelMenuBar *menubar)
+{
+        g_return_val_if_fail (PANEL_IS_MENU_BAR (menubar), 0);
+
+        return menubar->priv->orientation;
+}
